8 Flexible Parametric Survival, Simulation

* Unicode examples: Alt 8 ◘, Alt 22 ▬,  Alt 26 →,  Alt 27 ←, Alt 30 ▲, Alt 31 ▼    						
*◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘ I ◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘
*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ I ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬

local OBS "download programs listed in 0 Flexible Parametric Survival, Downloads"

* Current stable version of survsim
ssc install survsim, all replace

* The current development version of survsim in Stata can be installed by typing:
net install survsim, from(https://www.mjcrowther.co.uk/code/survsim)

* Update all installed adofiles that have newer versions
adoupdate		 					// pick up any updates.
ado update, update					// update

* Help file
help survsim

/*
Survsim can be used in four different ways:

  survsim parametric - a parametric distribution including the exponential, Gompertz and Weibull, and 2-component mixtures of them. 

  survsim user - a user-defined distribution. Survival times can be simulated from bespoke, user-defined [log] [cumulative] hazard functions. The function
            must be specified in Mata code (using colon operators).

  survsim model - a fitted merlin model. merlin fits a broad class of survival models, including standard parametric models, spline-based survival models,
            and user-defined survival models.

 survsim msm - a competing risks or general multi-state model. Event times can be simulated from transition-specific hazards, where each transition hazard
            function can be a standard parametric distribution, or a user-defined complex hazard function. Covariates and time-dependent effects can be specified for
            each transition-specific hazard independently.

  Baseline covariates can be included, with specified associated log hazard ratios.  Non-proportional hazards can also be included with all models.

*/


*◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘ Ordinary Survival Data ◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘

*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ User defined baseline ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
* A user defined basline hazard is typically a polynomial in t. This allows for many baseline hazard shapes. 
* You can also scale the hazard up or down to controll the overall number of events (deaths)
* Here are a few examples

* Background data. Run first!
clear
set seed 12345																	// seed
set obs 10000																	// N
global tmax=10																	// max follow up
gen trt=rbinomial(1,0.3)														// 30% treated
range t2 0 $tmax 101															// new time scale for plots
lab var t2 "time"	

* Hazards function plot	
tw	(function y=1 +1.2*x                  , range(0 $tmax)), title("Hazard function 1") name("f1", replace)
tw	(function y=15-1.2*x                  , range(0 $tmax)), title("Hazard function 2") name("f2", replace)
tw	(function y=1 +1.2*x-0.2*x^2+0.009*x^3, range(0 $tmax)), title("Hazard function 3") name("f3", replace)
tw	(function y=3 -1.2*x+0.2*x^2-0.009*x^3, range(0 $tmax)), title("Hazard function 4") name("f4", replace)
graph combine f1 f2 f3 f4

* Hazards functions	in Mata code using colon operators, sum is written :+, product is :*, exponetial is :^2
* The first number in the hazards is the scale. Change it up or down to set the total number of events (deaths)
global hf1 	0.004	:*(1  :+1.2:*{t})											// linear increasing														
global hf2 	0.004	:*(15 :-1.2:*{t})											// linear decreasing		
global hf3 	0.006	:*(1  :+1.2:*{t} :-0.2*{t}:^2 :+0.009:*{t}:^3)				// increasing upt to 5 years, then dropping   to half
global hf4 	0.006	:*(3  :-1.2:*{t} :+0.2*{t}:^2 :-0.009:*{t}:^3)				// decreasing upt to 5 years, then increasing to half

* Single state simulations (Run background data first)
capture drop T D h0																// drop if already present
survsim T D, hazard($hf4) maxtime($tmax) covariates(trt `=log(0.5)')			// Survival times T, event indicator D, treatment effect OR=0.5
stset T, fail(D==1)																// set set the data
stpm2 trt, scale(hazard) df(3) eform											// flexible parametric model with one covariate, 3 splines for the baseline
predict h0, haz zeros timevar(t2)												// predict basline hazards on the t2 timescale
tw (line h0 t2, col(blue))														// plot of the hazards
tab D																			// number of failures (deaths)



*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Weibull baseline ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
/* Weibull
For shape (gamma)<1, hazard is decreasing
For shape (gamma)>1, hazard is increasing

For shape (gamma)=1.2 and scale(lambda):
	Scale	%died
	0.01	5%
	0.05	24%
	0.1		42%
	0.2		66%
	0.3		80%
For shape (gamma) and scale(lambda)=0.1:
	shape	%died
	1.1		37%
	1.2		42%
	1.3		47%
	1.4		53%
*/

*================== Proportional Hazards Model =================================
* One covariate. Right cencoring at 10 years
#delimit ;
clear; set seed 498093; set obs 10000;																// N=10 000;
gen trt = runiform()>0.5; global λ=0.1; global γ=1.4; global mt=10;		   							// 50% treated. Weibull parameters, maxtime=10 years;
survsim stime died, dist(weibull) lambda($λ) gamma($γ) maxtime($mt) covariates(trt -0.5);			// treatment with HR=exp(-0.5)=0.6;
* Simple model to show the baseline hazard;
stset stime, f(died=1);	 
sts graph, by(trt) name("KM", replace);																// Kaplan-Meier;
stpm2 trt, scale(hazard) df(2); 																	// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(trt 0.5) timevar(t2); global d: display %3.1f 100*f[2];		// predict failure (deaths);
tw (line h0 t, col(blue)) (function y=$λ*$γ*x^($γ-1),range(0 $mt) col(green)), leg(off) note("Weibull, λ=$λ, γ=$γ, dead=$d%") name("h0", replace);
#delimit ;

* One covariate. Random cencoring times (0-10)
#delimit ;
clear; set seed 498093; set obs 10000;																// N=10 000;
gen trt = runiform()>0.5; global λ=0.1; global γ=1.4; global mt=10;		   							// 50% treated. Weibull parameters, maxtime=10 years;
gen censtime = runiform() * $mt;																	// random cencoring time;
survsim stime died, dist(weibull) lambda($λ) gamma($γ) maxtime(censtime) covariates(trt -0.5);		// treatment with HR=exp(-0.5)=0.6;
* Simple model to show the baseline hazard;
stset stime, f(died=1);	 
sts graph, by(trt) name("KM", replace);																// Kaplan-Meier;
stpm2 trt, scale(hazard) df(2); 																	// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(trt 0.5) timevar(t2); global d: display %3.1f 100*f[2];		// predict failure (deaths);
tw (line h0 t, col(blue)) (function y=$λ*$γ*x^($γ-1),range(0 $mt) col(green)), leg(off) note("Weibull, λ=$λ, γ=$γ, dead=$d%") name("h0", replace);
#delimit ;




*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Gompertz baseline ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
/* scale(λ), shape (γ)
λ		γ		%died	h0 at 0		2		10		type
.003	.038	3%		high 		min 	medium	UL
.01		.001	8%		high 		min 	medium	UL
.02		.001	15%		high 		min 	medium	UL
.03		.001	22%		high 		min 	medium	UL

.05		.001	33%		high 		medium 	min		decreasing

.001	.08		1%		low			medium	high	increasing

.001	.05		1%		low			max		high	inv UL
.001	.01		1%		low			max		medium	inv UL
.002	.05		2%		low			max		high	inv UL

*/

* One covariate. Right cencoring at 10 years
#delimit ;
clear; set seed 498093; set obs 10000;																// N=10 000;
gen trt = runiform()>0.5; global λ=0.002; global γ=0.05; global mt=10;		   						// 50% treated. maxtime=10 years;
survsim stime died, dist(gompertz) lambda($λ) gamma($γ) maxtime($mt) covariates(trt -0.5);			// treatment with HR=exp(-0.5)=0.6;
* Simple model to show the baseline hazard;
stset stime, f(died=1);	 
sts graph, by(trt) name("KM", replace);																// Kaplan-Meier;
stpm2 trt, scale(hazard) df(2); 																	// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(trt 0.5) timevar(t2); global d: display %3.1f 100*f[2];		// predict failure (deaths);
tw (line h0 t, col(blue)), leg(off) note("Gompertz, λ=$λ, γ=$γ, dead=$d%") name("h0", replace);
#delimit ;  // Conclusion: inverted U-L shape, 2% dead

#delimit ;
clear; set seed 498093; set obs 10000;																// N=10 000;
gen trt = runiform()>0.5; global λ=0.003; global γ=0.038; global mt=10;		   						// 50% treated, maxtime=10 years;
survsim stime died, dist(gompertz) lambda($λ) gamma($γ) maxtime($mt) covariates(trt -0.5);			// treatment with HR=exp(-0.5)=0.6;
* Simple model to show the baseline hazard;
stset stime, f(died=1);	 
sts graph, by(trt) name("KM", replace);																// Kaplan-Meier;
stpm2 trt, scale(hazard) df(2); 																	// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(trt 0.5) timevar(t2); global d: display %3.1f 100*f[2];		// predict failure (deaths);
tw (line h0 t, col(blue)), leg(off) note("Gompertz, λ=$λ, γ=$γ, dead=$d%") name("h0", replace);
#delimit ;	// Conclusion: U-L shape, 3% dead


#delimit ;
clear; set seed 498093; set obs 10000;
gen trt = runiform()>0.5; global λ=0.003; global γ=0.05;				   						// 50% treated; Gompertz parameters;
survsim stime died, dist(gompertz) lambda($λ) gamma($γ) maxtime(5) covariates(trt -0.5);		// treatment with HR=exp(-0.5)=0.6;
* Describe data;
sum died; global d: display %3.1f 100*r(mean); dis $d; #delimit cr 
// Simple model to show the baseline hazard
stset stime, f(died=1)	 
stpm2 trt, scale(hazard) df(2) 
range t 0 5 101
predict h0, haz zeros timevar(t)
tw (line h0 t, col(blue)), note("Gompertz, λ=$λ, γ=$γ, dead=$d%") name("h0", replace)

#delimit ;
clear; set seed 498093; set obs 10000;
gen trt = runiform()>0.5; global λ=0.003; global γ=0.03;				   						// 50% treated; Gompertz parameters;
survsim stime died, dist(gompertz) lambda($λ) gamma($γ) maxtime(5) covariates(trt -0.5);		// treatment with HR=exp(-0.5)=0.6;
* Describe data;
sum died; global d: display %3.1f 100*r(mean); dis $d; #delimit cr 
// Simple model to show the baseline hazard
stset stime, f(died=1)	 
stpm2 trt, scale(hazard) df(2) 
range t 0 5 101
predict h0, haz zeros timevar(t)
tw (line h0 t, col(blue)), note("Gompertz, λ=$λ, γ=$γ, dead=$d%") name("h0", replace)



#delimit ;
clear; set seed 498093; set obs 10000;
gen trt = runiform()>0.5; global λ=0.01; global γ=0.05;				   						// 50% treated; Gompertz parameters;
survsim stime died, dist(gompertz) lambda($λ) gamma($γ) maxtime(5) covariates(trt -0.5);		// treatment with HR=exp(-0.5)=0.6;
* Describe data;
sum died; global d: display %3.1f 100*r(mean); dis $d; #delimit cr 
// Simple model to show the baseline hazard
stset stime, f(died=1)	 
stpm2 trt, scale(hazard) df(2) 
range t 0 5 101
predict h0, haz zeros timevar(t)
tw (line h0 t, col(blue)), note("Gompertz, λ=$λ, γ=$γ, dead=$d%") name("h0", replace)




//  (function y=$γ*exp(x*$λ),range(0 5) col(green) lw(*2))





−1 + 0.02t − 0.03t2 + 0.005t3
tw (function y=-1+0.02*x-0.03*x^2+0.005*x^3, range(0 2))
tw (function y=1+0.09*x-0.018*x^2+0.0009*x^3, range(0 10))


*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Polynomial baseline ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
* First a baseline hazard function (h) is gived as values (from 0-1) for t=0-10
* Then a fractional polynomial is fitted to the baseline hazard data
* Then this functional form is used in survsim (possibly scaled down) or usinf a constant term controll proportion of events

* Describing a baseline hazard function (h) as values (from 0-1) for t=0-10
* Inverse UL shape
#delimit ;
clear; set seed 498093; set obs 10000;
gen t=runiformint(0,10);
gen     h=.3 	if t==0;
replace h=.35 	if t==1;
replace h=.5 	if t==2;
replace h=.9 	if t==3;
replace h=.9 	if t==4;
replace h=.8 	if t==5;
replace h=.6 	if t==6;
replace h=.55 	if t==7;
replace h=.5 	if t==8;
replace h=.5 	if t==9;
replace h=.5 	if t==10;
line h t, sort title("Baseline hazard function");
#delimit cr

* Fitting a fractional polynomial with fixed powers to the (h,t)-data
#delimit ;
glo a=5; glo b=1;					// t is scaled to (t+a)/b. Scaled t can not contain or be close to 0 and should not be to large;
*glo pow "-2 -1 0 0.5 1 2 3";													// list of powers (must be ordered);
glo pow "1 2 3";													// list of powers (must be ordered);
fp <t>, fp($pow) scale($a $b) replace: regress h <t>;							// fractional polynomial regression in t;
cap drop hfp; loc j=0;
foreach p of global pow	{;
  loc ++j; 
  loc f=cond(`p'==0,"ln((x+$a)/$b)","((x+$a)/$b)^`p'");							// powers of scaled t, 0 means log(t)
  loc ys "`ys' `pl' _b[t_`j']*t_`j'";											// build up fractional polynomial from generated variables and coefficients;
  loc fs "`fs' `pl' _b[t_`j']*`f'";												// build up fractional polynomial function;
  loc pl +;																		// add a plus sign to the sum of terms;
  glo b`j'=_b[t_`j'];															// save coefficients for later use
};
glo n=`j'; glo b0=_b[_cons];													// number og power terms and constant term;
gen hfp=`ys'+ _b[_cons];														// fractional polynomial from generated variables and coefficients;
tw	(line h t, sort)(line hfp t, sort)											/* given hazard and fp fit */ 
	(function y=`fs'+ _b[_cons], range(t)), leg(off);							// fp fit as function;
#delimit cr	

* Simulating survival data based on the fitted fractional polynomial
* Scale polynomial terms down to reduce overall events. Also find signs
glo c=12.5									// scaling factor to controll overall events, higher c → fewer events 
forvalues j=0(1) $n {
  glo c`j'=${b`j'}/$c
  glo s`j'=cond(${c`j'}>0,"+","")			// signs for the poly terms
  dis "${s`j'}${c`j'}"
}
tw	(function y=$c0+$c1*(x+$a)+$c2*(x+$a)^2+$c3*(x+$a)^3, range(0 10)), name("p1", replace)

#delimit ;
set seed 12345;
foreach v in E T D t t2 h0 f {; cap drop `v'; };								// drop variables;
gen E=rbinomial(1,0.5); loc e=log(2);											// exposure E with HR
survsim T D, hazard($c0 :$s1$c1:*({t}:+$a) :$s2$c2:*({t}:+$a):^2 :$s3$c3:*({t}:+$a):^3) covariates(E `e') maxtime(10);
* Simple model to show the baseline hazard;
glo mt=10;																							// max time;
stset T, f(D=1);	 
sts graph, by(E) name("KM", replace);																// Kaplan-Meier;
stcox E;
stcurve, hazard at(E=0) name("Coxh0", replace);
stpm2 E, scale(hazard) df(4); 																		// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(E 0.5) timevar(t2); global d: display %3.1f 100*f[2];			// predict failure (deaths) at end;
tw (line h0 t, col(blue)), leg(off) note("Polynomial, scale=$c, dead=$d %") name("h0", replace);
#delimit ;  // Conclusion: Inverted UL shape, 50% dead

* UL shape
replace h=10-h
sort t
line h t, sort title("Baseline hazard function")

* Fitting a fractional polynomial with fixed powers to the (h,t)-data
#delimit ;
glo a=5; glo b=1;					// t is scaled to (t+a)/b. Scaled t can not contain or be close to 0 and should not be to large;
*glo pow "-2 -1 0 0.5 1 2 3";													// list of powers (must be ordered);
glo pow "1 2 3";													// list of powers (must be ordered);
fp <t>, fp($pow) scale($a $b) replace: regress h <t>;							// fractional polynomial regression in t;
cap drop hfp; loc j=0;
foreach p of global pow	{;
  loc ++j; 
  loc f=cond(`p'==0,"ln((x+$a)/$b)","((x+$a)/$b)^`p'");							// powers of scaled t, 0 means log(t)
  loc ys "`ys' `pl' _b[t_`j']*t_`j'";											// build up fractional polynomial from generated variables and coefficients;
  loc fs "`fs' `pl' _b[t_`j']*`f'";												// build up fractional polynomial function;
  loc pl +;																		// add a plus sign to the sum of terms;
  glo b`j'=_b[t_`j'];															// save coefficients for later use
};
glo n=`j'; glo b0=_b[_cons];													// number og power terms and constant term;
gen hfp=`ys'+ _b[_cons];														// fractional polynomial from generated variables and coefficients;
tw	(line h t, sort)(line hfp t, sort)											/* given hazard and fp fit */ 
	(function y=`fs'+ _b[_cons], range(t)), leg(off);							// fp fit as function;
#delimit cr	

* Simulating survival data based on the fitted fractional polynomial
* Scale polynomial terms down to reduce overall events. Also find signs
glo c=190									// scaling factor to controll overall events, higher c → fewer events 
forvalues j=0(1) $n {
  glo c`j'=${b`j'}/$c
  glo s`j'=cond(${c`j'}>0,"+","")			// signs for the poly terms
  dis "${s`j'}${c`j'}"
}
tw	(function y=$c0+$c1*(x+$a)+$c2*(x+$a)^2+$c3*(x+$a)^3, range(0 10)), name("p1", replace)

#delimit ;
set seed 12345;
foreach v in E T D t t2 h0 f {; cap drop `v'; };								// drop variables;
gen E=rbinomial(1,0.5); loc e=log(2);											// exposure E with HR
survsim T D, hazard($c0 :$s1$c1:*({t}:+$a) :$s2$c2:*({t}:+$a):^2 :$s3$c3:*({t}:+$a):^3) covariates(E `e') maxtime(10);
* Simple model to show the baseline hazard;
glo mt=10;																							// max time;
stset T, f(D=1);	 
sts graph, by(E) name("KM", replace);																// Kaplan-Meier;
stcox E;
stcurve, hazard at(E=0) name("Coxh0", replace);
stpm2 E, scale(hazard) df(4); 																		// Flexible parametric model;
range t 0 $mt 101; predict h0, haz zeros timevar(t);												// predict baseline hazard;
range t2 0 $mt 2; predict f, fail at(E 0.5) timevar(t2); global d: display %3.1f 100*f[2];			// predict failure (deaths) at end;
tw (line h0 t, col(blue)), leg(off) note("Polynomial, scale=$c, dead=$d %") name("h0", replace);
#delimit ;  // Conclusion: 




*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Mixed baseline ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬






































*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Data with three covariates ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// Simulate simple data with 3 covariate
#delimit ;
clear; set seed 498093; set obs 10000;
gen trt = runiform()>0.5; 	global β1=log(0.6);									// 50% treated, HR=0.6;
gen sex = runiform()>0.5; 	global β2=log(1.5);									// 50% females, HR=1.5;
gen age = runiform(20,80);	global β3=log(1.01);								// 20 to 80 years, HR=1.01;
global λ=0.05;  global γ=1.2;													// scale and shape;
survsim stime died, dist(weibull) lambda($λ) gamma($γ) maxtime(5) covariates(trt $β1  sex $β2  age $β3);
# delimit cr

// Describe data
tab died
tab trt died, row nofreq

// Estimation
stset stime, f(died=1)	 
stpm2 trt sex age, scale(hazard) df(1) eform

// Prediction
range t 0 5 101
predict h0, haz zeros timevar(t)
tw (line h0 t, col(blue)), name("h0", replace)


*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Time varying effects ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// Time varying effect of sex
#delimit ;
clear; set seed 498093; set obs 10000;
gen trt = runiform()>0.5; 	global β1=log(0.6);									// 50% treated, HR=0.6;
gen sex = runiform()>0.5; 	global β2=log(1.5);									// 50% females, HR=1.5;
gen age = runiform(20,80);	global β3=log(1.01);								// 20 to 80 years, HR=1.1;
global β4=log(1.3);																// Time dependent effect of sex;
global λ=0.05;  global γ=1.2;													// scale and shape;
survsim stime died, dist(weibull) lambda($λ) gamma($γ) maxtime(5) covariates(trt $β1  sex $β2  age $β3) tde(sex $β4);
# delimit cr

// Describe data
tab died
tab trt died, row nofreq

// Estimation
stset stime, f(died=1)	 
stpm2 trt sex age, scale(hazard) df(1) tvc(sex) dftvc(1) eform

// Prediction
range t 0 5 101
predict hr, hrnumerator(trt 0 sex 1 age 60) hrdenominator(trt 0 sex 0 age 60) timevar(t)  ci					// HR
* Plot all HRs (without CI)
tw 	(line hr t, sort)

*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Competing risk model ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
* Two competing risks simulated from two different Weibull models
set seed 6765327
clear
set obs 10000
generate trt = rbinomial(1,0.5)
survsim stime event, distribution(weibull) cr ncr(2) lambdas(0.1 0.1) gammas(1.5 0.5) covariates(trt -0.5 0.5)
replace event = 0 if stime>15
stset stime, failure(event==1)

*◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘ Repeated Measures and Survival Data ◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘
*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Two level random intercept, current value ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// Two level linear random intercept model, outcome y. Survival model dependent on current level of y
// 1000 subjects with 10 measures each

/* Equation: 
	yij = α0 + ai + α1c1+...
	   t*(β0 + bi + β1v1+...) + εij
   α0, β0 	intercept and t-slope
   ai, bi 	random intercept and  random t-slope
   c1, c2	covariates for y at first measure
   v1, v2	covariates for change in y over time
   
   surv(d)=γ0 + γ1ai + γ2bi + γ3c1 + γ4v1
*/

#delimit ;
clear; global subjects=1000; global measures=10; global maxt=$measures;			// level 2: 1000 subjects. level 1: 10 measures ;
set seed 12345; set obs $subjects; 				 						
* Level 2;
gen id=_n; gen a=rnormal(0,1); gen b=rnormal(0,1);  							// random intercept and slope ; 	
global α0=5; global α1=-2; gen c1=runiform()>0.5; 								// α, intercept and baseline covariates with effects;
* Level 1;
expand $measures; sort id;	by id: gen t=_n-1;  								// level 1: 10 measures per subject, t=0-9 for all;
global β0=0.5; global β1=0; gen v1=runiform()>0.2;								// β, slope and covariates with effects for change over time;
gen y=$α0 +a +$α1*c1	 +t*($β0 + b +$β1*v1)  	+runiform(0,3);					// continuous repeated y ;
* Survival;
global λλ=0.3; global γγ=1.5;													// parameters for Weibull;
global γ1=-0.3; global γ2=0; global γ3=0; global γ4=-0.1; global γ5=-0.2;		// effects for: c1, v1, EV[y], random intercept, random slope ;
survsim time event, dist(weibull) lambda($λλ) gamma($γγ) maxtime($maxt) covariates(c1 $γ1 v1 $γ2   y $γ3 a $γ4 b $γ5);   

* The simulation above is for recurrent events. Change to single event;
#delimit ;
gen start=t; 	gen stop=t+1; gen delta=stop-start; gen died=(time<delta);
replace died=. if (died[_n-1]==1 | died[_n-1]==.) & id==id[_n-1]; drop if died==.;
replace stop=start+time if died==1; gen stime=stop if died==1; drop time event delta;
* Time and died for Merlin;
by id: egen maxdied=max(died); by id: egen numb=count(id); 
gen stime2=.; 
replace stime2=stime[_n+numb[_n]-1] 	if id!=id[_n-1] ;
replace stime2=$maxt                   	if id!=id[_n-1] & maxdied==0;
gen died2=.;
replace died2=maxdied					if id!=id[_n-1] ;
drop maxdied numb stime;
list  if inlist(id,1,2,4,6,9), sepby(id);
tab died;

// Describe results;
#delimit ;
tw (kdensity y), name("dens",replace);
preserve; keep if id<=12; tw (line y t,con(asc)), by(id) name("g2",replace); restore;
tab died; #delimit cr



// Stsetting the data, needed for stjm, stjmgraph also need enter in the stset, stset not needed for merlin
stset stop, enter(start)  f(died=1) id(id)
list id _t0 _t _d y stime if inlist(id,1,2,4,6,9), sepby(id)

// Result matrix
mat R=(.\ $α1 \ $β0 \ $β1 \ $α0 \.\ $γ1 \ $γ2\ $γ3 \ $γ4 \ $γ5)
mat rowname R="Longitudinal: $subjects*$measures" "c1" "t" "t*v1" "cons" "Survival" "c1" "v1" "EV[y]" "M1" "M2" 
mat colname R= "true"
mat list R

* Mean value of the exposure over time for censored and event (died)
*stjmgraph y, panel(id) lowess

// Linear mixed model using mixed
mixed y c1 t c.t#v1 || id: t
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[4,1] \g[5,1] \.\.\.\.\.\.)
mat colname R="" "mixed"
mat list R, format(%5.2f)

// Linear mixed model using merlin
merlin 	(y c1 t t#v1 M1[id]@1 t#M2[id]@1, family(gaussian)  timevar(t))
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[3,1] \g[6,1] \.\.\.\.\.\.)
mat colname R="" "" "merlin: mixed"
mat list R, format(%5.2f)

// Logistic model
logit died2 c1 v1
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\g[2,1]\.\.\.)
mat colname R="" "" "" "logit"
mat list R, format(%5.2f)

// Merlin: Logistic
merlin 	(died2 c1 v1, family(bernoulli))
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\g[2,1]\.\.\.)
mat colname R="" "" "" "" "merlin: logit"
mat list R, format(%5.2f)

// Joint mixed and logistic
merlin 	(y c1 t t#v1 M1[id]@1 t#M2[id]@1, family(gaussian)  timevar(t))			///		/* c1 and t plus random intercept and random slope */
		(died2 c1 v1 M1[id] M2[id], family(bernoulli))							/* c1, v1 plus current value, random intercept and random slope */
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[3,1] \g[6,1] \. \g[8,1] \g[9,1] \. \g[10,1] \g[11,1])
mat colname R="" "" "" "" "" "merlin: mix-logit"
mat list R, format(%5.2f)

// Weibull regression
streg c1 v1, distr(weibull) nohr
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\g[2,1]\.\.\.)
mat colname R="" "" "" "" "" "" "Weibull"
mat list R, format(%4.2f)

// Merlin: Weibull regression
merlin 	(stime2 c1 v1, family(weibull, failure(died2)) timevar(stime2))	/* treatment and current value of y as covariates */
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\g[2,1]\.\.\.)
mat colname R="" "" "" "" "" "" "" "merlin: Weibull"
mat list R, format(%4.2f)

// Merlin: JM
merlin 	(y c1 t t#v1 M1[id]@1 t#M2[id]@1	, family(gaussian)  timevar(t))			///
		(stime2 c1 v1  M1[id] M2[id]	, family(weibull, failure(died2)) timevar(stime2))	
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[3,1] \g[6,1] \. \g[8,1] \g[9,1] \. \g[10,1] \g[11,1] )
mat colname R="" "" "" "" "" "" "" "" "merlin: JM"
mat list R, format(%4.2f)



*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Starting values ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// use starting values for merlin from stjm
stjm y trt , panel(id) survm(weibull) ffp(1) survcov(trt)
mat S=e(b)				// startvalues

// merlin with standard startvalues
set rmsg on																		// measure estimation time
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))					///	/* trt and t plus random intercept */
		(stime2 trt EV[y]	, family(weibull, failure(died2)) timevar(stime2))	/* treatment and current value of y as covariates */
* Conclusion: 21 sec

// merlin with startvalues from stjm
set rmsg on																				// measure estimation time
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))						///		/* trt and t plus random intercept */
		(stime2 trt EV[y]	, family(weibull, failure(died2)) timevar(stime2))	///		/* treatment and current value of y as covariates */
		, from(S)
* Conclusion: 11 sec

*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Using merlin for all models ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// Linear mixed model
mixed y trt t || id:
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))

// Weibull regression
streg trt y, distr(weibull) nohr
merlin 	(stime2 trt y	, family(weibull, failure(died2)) timevar(stime2))

// Current value JM using the expected value (EV) of y in the survival model, merlin		
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))					///	/* trt and t plus random intercept */
		(stime2 trt EV[y]	, family(weibull, failure(died2)) timevar(stime2))	/* treatment and current value of y as covariates */



		
*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Three level survival data ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
/*
Analysis of clustered survival data with three levels. Such data arises in the meta-analysis of recurrent event data, where we have events, 
k, nested within patients, j, nested within trials, i.
Random intercepts
The first example will consider a model with a random intercept at level 1 and a random intercept at level 2,
First we must be clear about what assumptions this model has. By assuming a random intercept at the trial level 1, we are assuming that each trial comes 
from a distribution of trials, which arguably is a strong and perhaps implausible assumption. We may prefer to stratify by trial, but for the sake of illustration
, here I’ll proceed with a random trial effect. A second important note is that our covariate effects, 
β, are interpreted as conditonal on the random effects.
*/

clear
set seed 498093
set obs 15							// level 3: 15 trials 
gen trial = _n
gen b1 = rnormal(0,1)				// level 3 random intercept			

expand 100							// level 2: 100 patients in each trial
sort trial
gen id = _n							// patient id
gen b2 = rnormal(0,1)				// level 2 random intercept
gen trt = runiform()>0.5			// 50% treated

expand 2							// level 1: up to two events
sort trial id

// Weibull baseline haxard with lambda=0.1 and gamma=1.2, and a log hazard ratio of beta=-0.5 for the treatment effect
global a=0.1
global b=1.2
survsim stime died, dist(weibull) lambda($a) gamma($b) maxtime(5) covariates(trt -0.5 b1 1 b2 1)
// Note: simulating the recurrent events under a clock-reset approach, i.e. after each event the timescale is reset to 0. 

tab died

// Simple model to show the baseline hazard
stset stime, f(died=1) id(id)	 
stpm2 trt, scale(hazard) df(1) 
range t 0 5 101
cap drop h0
predict h0, haz zeros timevar(t)
tw (line h0 t, col(blue))  (function y=weibullph($a,$b,x),range(0 5) col(green) lw(*2)), leg(off) note("a=$a, b=$b") name("h0", replace)




tw	(function y=weibullph(.1,1.2,x),range(0 2.5) ), name("Wei",replace)

// True three level model
merlin (stime trt M1[trial]@1 M2[trial>id]@1, family(weibull, failure(died))) 

/*
Random trial and random treatment effect
Now we consider a second example, but still sticking within a meta-analysis context. 
In many situations, we would investigate the presence of heterogeneity, i.e. unobserved variability, in any treatment effects. 
We can do this by modelling a random treatment effect, with the following model.
*/

clear
set seed 498093
set obs 15							// level 3: 15 trials 
gen trial = _n
gen b11 = rnormal(0,1)				// level 3 random intercept			
gen b12 = rnormal(0,1)				// level 3 random treatment effect			

expand 100							// level 2: 100 patients in each trial
sort trial
gen id = _n							// patient id
gen b2 = rnormal(0,1)				// level 2 random intercept
gen trt = runiform()>0.5			// 50% treated

expand 2							// level 1: up to two events
sort trial id
gen trtb12 = (-0.5 + b12) * trt		// overall treatment effect log beta=-0.5 for treatment

survsim stime died, dist(weibull) lambda(0.1) gamma(1.2) maxtime(5) covariates(trtb12 1 b11 1 b2 1)	// notice all covariates added with coefficient 1

merlin (stime trt trt#M1[trial]@1 M2[trial]@1 M3[trial>id]@1, family(weibull, failure(died))) 
* Conclusion: sd(M1)=1.12 (0.76,1.65) The sd is lager than 0 indicating heterogeniety in the treatment effect over trials





*◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘ Repeated measures and Logistic model ◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘
*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Data: Two level random intercept and slope ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬
// Two level linear random intercept and slope model, outcome y. Logistic model dependent on both random terms
// 1000 subjects with 10 measures each
/* Equation: 
	yij = α0 + ai + α1c1+...
	   t*(β0 + bi + β1v1+...) + εij
   α0, β0 	intercept and t-slope
   ai, bi 	random intercept and  random t-slope
   c1, c2	covariates for y at first measure
   v1, v2	covariates for change in y over time
   
   logit(d)=γ0 + γ1ai + γ2bi + γ3v1
*/

#delimit ;
global subjects=1000; global measures=10; 										// level 2: 1000 subjects. level 1: 10 measures ;
clear; set seed 12345; set obs $subjects; 				 						
gen id=_n; gen a=rnormal(0,1); gen b=rnormal(0,1);  							// random intercept and slope ; 	
global α0=5; global α1=2; gen c1=runiform()>0.3; 								// intercept and covariates with effects;

expand $measures; sort id;	by id: gen t=_n-1;  								// level 1: 10 measures per subject, t=0-9 for all;
global β0=2; global β1=-1; gen v1=runiform()>0.2;								// slope and covariates with effects for change over time
gen y=$α0 +a +$α1*c1	 +t*($β0 + b +$β1*v1)  	+runiform(0,3);				// continuous repeated y. ;

global γ0=0.1; global γ1=-0.1; global γ2=-0.2; global γ3=0;
gen p=$γ1*a + $γ2*b + $γ3*v1 + rlogistic(log($γ0/(1-$γ0)),1) if t==0;
gen d=p>0 if t==0;																// binary d, depending on both random intercept and slope;

* Describe the data;
list  if inlist(id,1,2,3), sepby(id);
sum p d;
tw (kdensity p), xline(0);

// Individual growth plots
preserve
keep if id<=12
tw (line y t) ,by(id)
restore

// Result matrix
mat R=(.\ $α1 \ $β0 \ $β1 \ $α0 \.\ $γ3 \ $γ1 \ $γ2)
mat rowname R="Logitudinal" "c1" "t" "t*v1" "cons" "Survival" "v1" "M1" "M2" 
mat list R


// Linear mixed model using mixed
mixed y c1 t c.t#v1 || id: t
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[4,1] \g[5,1] \.\.\.\.)
mat colname R="True" "mixed"
mat list R, format(%5.2f)

// Linear mixed model using merlin
merlin 	(y c1 t t#v1 M1[id]@1 t#M2[id]@1, family(gaussian)  timevar(t))
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[3,1] \g[6,1] \.\.\.\.)
mat colname R="True" "mixed" "merlin: mixed"
mat list R, format(%5.2f)

// Logistic model
logit d v1
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\.\.)
mat colname R="True" "mixed" "merlin:mixed" "logit"
mat list R, format(%5.2f)

merlin 	(d v1, family(bernoulli))
mat g=e(b)'
mat R=R,(. \. \. \. \. \.\g[1,1]\.\.)
mat colname R="True" "mixed" "merlin:mixed" "logit" "merlin: logit"
mat list R, format(%5.2f)


// Joint mixed and logistic
merlin 	(y c1 t t#v1 M1[id]@1 t#M2[id]@1, family(gaussian)  timevar(t))			///		/* c1 and t plus random intercept and random slope */
		(d v1 M1[id] M2[id], family(bernoulli))											/* v1 plus random intercept and random slope */
mat g=e(b)'
mat R=R,(. \g[1,1] \g[2,1] \g[3,1] \g[6,1] \. \g[8,1] \g[9,1] \g[10,1])
mat colname R="True" "mixed" "merlin: mixed" "logit" "merlin: logit" "merlin: mix-logit"
mat list R, format(%5.2f)

		
		



// Weibull regression
streg trt y, distr(weibull) nohr
merlin 	(stime2 trt y	, family(weibull, failure(died2)) timevar(stime2))

// Current value JM using the expected value (EV) of y in the survival model, merlin		
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))					///	/* trt and t plus random intercept */
		(stime2 trt EV[y]	, family(weibull, failure(died2)) timevar(stime2))	/* treatment and current value of y as covariates */





*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ Linear mixed model ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬


// Linear mixed model
mixed y trt t || id:
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))

// Weibull regression
streg trt y, distr(weibull) nohr
merlin 	(stime2 trt y	, family(weibull, failure(died2)) timevar(stime2))

// Current value JM using the expected value (EV) of y in the survival model, merlin		
merlin 	(y trt t M1[id]@1 	, family(gaussian)  timevar(t))					///	/* trt and t plus random intercept */
		(stime2 trt EV[y]	, family(weibull, failure(died2)) timevar(stime2))	/* treatment and current value of y as covariates */


*◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘ I ◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘◘
*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬ I ▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬











